@@ -54,14 +54,6 @@ OUTGOING_TAG: str = 'out'
 INCOMING_TAG: str = 'in'
 TENANT_RIGHTS_TAG: str = 'tenantRights'
 
-# Mapping to map the simple language_code code to a default language_code / country code
-language_code_mapping: Dict[str, LanguageCode] = {
-    'en': LanguageCode('en_US'),
-    'ja': LanguageCode('ja_JP'),
-    'de': LanguageCode('de_DE'),
-    'bg': LanguageCode('bg_BG')
-}
-
 
 class EntityStatus(enum.Enum):
     """
