@@ -3,11 +3,9 @@
 from json import JSONEncoder
 from typing import List, Union, Set, Tuple
 
-from knowledge import logger
 from knowledge.base.access import TenantAccessRight
 from knowledge.base.entity import *
 
-
 # ---------------------------------------------- Vocabulary base URI ---------------------------------------------------
 PREFIX: str = "xsd"
 BASE_URI: str = "http://www.w3.org/2001/XMLSchema#"
@@ -1251,43 +1249,6 @@ class ThingObject(abc.ABC):
                f'tenant access right:={self.tenant_access_right}]>'
 
 
-def update_language_code(lang: str):
-    return language_code_mapping.get(lang, lang)
-
-
-def localized_list_description(entity_dict: Dict[str, str]) -> List[Description]:
-    return [Description(cont, update_language_code(lang)) for lang, cont in entity_dict.items()]
-
-
-def localized_list_label(entity_dict: Dict[str, str]) -> List[Label]:
-    return [Label(cont, update_language_code(lang), main=True) for lang, cont in entity_dict.items()]
-
-
-def localized_flatten_alias_list(entity_dict: Dict[str, List[str]]) -> List[Label]:
-    flatten: List[Label] = []
-    for language, items in entity_dict.items():
-        for i in items:
-            flatten.append(Label(i, update_language_code(language), main=False))
-    return flatten
-
-
-def from_dict(entity: Dict[str, Any], concept_type: OntologyClassReference) -> 'ThingObject':
-    labels: List[Label] = localized_list_label(entity['label'])
-    description: List[Description] = localized_list_description(entity['description'])
-    alias: List[Label] = localized_flatten_alias_list(entity['alias'])
-    if IMAGE_TAG in entity:
-        icon: str = entity[IMAGE_TAG]
-    else:
-        logger.warning(f"Entity has no image: {entity}")
-        icon: str = ''
-    # Create the entity
-    thing: ThingObject = ThingObject(label=labels, concept_type=concept_type, description=description, icon=icon)
-    thing.alias = alias
-    if STATUS_FLAG_TAG in entity:
-        thing.status_flag = entity[STATUS_FLAG_TAG]
-    return thing
-
-
 # -------------------------------------------------- Encoder -----------------------------------------------------------
 class ThingEncoder(JSONEncoder):
     def default(self, o):
