@@ -16,6 +16,7 @@ from knowledge.base.ontology import DataProperty, OntologyPropertyReference, Thi
 from knowledge.services.base import WacomServiceAPIClient, WacomServiceException
 
 # ------------------------------------------------- Constants ----------------------------------------------------------
+ACTIVATION_TAG: str = 'activation'
 SEARCH_TERM: str = 'searchTerm'
 LANGUAGE_PARAMETER: str = 'language'
 TYPES_PARAMETER: str = 'types'
@@ -60,11 +61,11 @@ class WacomKnowledgeService(WacomServiceAPIClient):
     ---------------------
     Client for the Semantic Ink Privat knowledge system.
 
-    Operations for concepts:
-        - Creation of concepts
-        - Update of concepts
-        - Deletion of concepts
-        - Listing of concepts
+    Operations for entitys:
+        - Creation of entitys
+        - Update of entitys
+        - Deletion of entitys
+        - Listing of entitys
 
     Parameters
     ----------
@@ -77,9 +78,9 @@ class WacomKnowledgeService(WacomServiceAPIClient):
     """
     PERSONAL_KNOWLEDGE_URL: str = 'https://semantic-ink-private.wacom.com'
     USER_ENDPOINT: str = 'user'
-    CONCEPT_ENDPOINT: str = 'entity'
+    ENTITY_ENDPOINT: str = 'entity'
     ACTIVATIONS_ENDPOINT: str = 'entity/activations'
-    DELETION_ENDPOINT: str = 'entity/{}'
+    ENTITY_URI_ENDPOINT: str = 'entity/{}'
     LABELS_ENDPOINT: str = 'entity/{}/labels'
     LISTING_ENDPOINT: str = 'entity/types'
     LITERAL_ENDPOINT: str = 'entity/{}/literals'
@@ -107,7 +108,7 @@ class WacomKnowledgeService(WacomServiceAPIClient):
         auth_key: str
             Auth key identifying a user within the Wacom personal knowledge service.
         uri: str
-            URI of concept
+            URI of entity
 
         Returns
         -------
@@ -119,7 +120,7 @@ class WacomKnowledgeService(WacomServiceAPIClient):
         WacomServiceException
             If the graph service returns an error code or the entity is not found in the knowledge graph
         """
-        url: str = f'{self.service_base_url}{WacomKnowledgeService.CONCEPT_ENDPOINT}/{urllib.parse.quote(uri)}'
+        url: str = f'{self.service_base_url}{WacomKnowledgeService.ENTITY_ENDPOINT}/{urllib.parse.quote(uri)}'
         headers: Dict[str, str] = {
             AUTHORIZATION_HEADER_FLAG: f'Bearer {auth_key}'
         }
@@ -155,7 +156,7 @@ class WacomKnowledgeService(WacomServiceAPIClient):
             else:
                 thing.tenant_access_right = TenantAccessRight()
             return thing
-        raise WacomServiceException(f'Pushing concept failed. '
+        raise WacomServiceException(f'Pushing entity failed. '
                                     f'Response code:={response.status_code}, exception:= {response.content}')
 
     def delete_entities(self, auth_key: str, uris: List[str], force: bool = False):
@@ -167,7 +168,7 @@ class WacomKnowledgeService(WacomServiceAPIClient):
         auth_key: str
             Auth key from user
         uris: List[str]
-            List of URI of concepts. **Remark:** More than 100 entities are not possible in one request
+            List of URI of entitys. **Remark:** More than 100 entities are not possible in one request
         force: bool
             Force deletion process
 
@@ -178,7 +179,7 @@ class WacomKnowledgeService(WacomServiceAPIClient):
         """
         if len(uris) > 100:
             raise WacomServiceException("Please delete less than 100 entities.")
-        url: str = f'{self.service_base_url}{WacomKnowledgeService.CONCEPT_ENDPOINT}'
+        url: str = f'{self.service_base_url}{WacomKnowledgeService.ENTITY_ENDPOINT}'
         headers: Dict[str, str] = {
             AUTHORIZATION_HEADER_FLAG: f'Bearer {auth_key}'
         }
@@ -188,7 +189,7 @@ class WacomKnowledgeService(WacomServiceAPIClient):
         }
         response: Response = requests.delete(url, headers=headers, params=params, verify=self.verify_calls)
         if not response.ok:
-            raise WacomServiceException(f'Deletion of concept failed.'
+            raise WacomServiceException(f'Deletion of entity failed.'
                                         f'Response code:={response.status_code}, exception:= {response.content}')
 
     def delete_entity(self, auth_key: str, uri: str, force: bool = False):
@@ -200,7 +201,7 @@ class WacomKnowledgeService(WacomServiceAPIClient):
         auth_key: str
             Auth key from user
         uri: str
-            URI of concept
+            URI of entity
         force: bool
             Force deletion process
 
@@ -209,14 +210,14 @@ class WacomKnowledgeService(WacomServiceAPIClient):
         WacomServiceException
             If the graph service returns an error code
         """
-        url: str = f'{self.service_base_url}{WacomKnowledgeService.DELETION_ENDPOINT.format(urllib.parse.quote(uri))}'
+        url: str = f'{self.service_base_url}{WacomKnowledgeService.ENTITY_URI_ENDPOINT.format(urllib.parse.quote(uri))}'
         headers: Dict[str, str] = {
             AUTHORIZATION_HEADER_FLAG: f'Bearer {auth_key}'
         }
         response: Response = requests.delete(url, headers=headers, params={'forceDelete': force},
                                              verify=self.verify_calls)
         if not response.ok:
-            raise WacomServiceException('Deletion of concept failed. Response code:={}, exception:= {}'
+            raise WacomServiceException('Deletion of entity failed. Response code:={}, exception:= {}'
                                         .format(response.status_code,
                                                 response.content))
 
@@ -242,16 +243,16 @@ class WacomKnowledgeService(WacomServiceAPIClient):
         except WacomServiceException:
             return False
 
-    def create_entity(self, auth_key: str, concept: ThingObject):
+    def create_entity(self, auth_key: str, entity: ThingObject):
         """
-        Creates concept in graph.
+        Creates entity in graph.
 
         Parameters
         ----------
         auth_key: str
             Auth key from user
-        concept: ThingObject
-            Concept object
+        entity: ThingObject
+            entity object
 
         Returns
         -------
@@ -263,7 +264,7 @@ class WacomKnowledgeService(WacomServiceAPIClient):
         WacomServiceException
             If the graph service returns an error code
         """
-        url: str = f'{self.service_base_url}{WacomKnowledgeService.CONCEPT_ENDPOINT}'
+        url: str = f'{self.service_base_url}{WacomKnowledgeService.ENTITY_ENDPOINT}'
         # Different localized content
         labels: List[dict] = []
         descriptions: List[dict] = []
@@ -274,7 +275,7 @@ class WacomKnowledgeService(WacomServiceAPIClient):
             AUTHORIZATION_HEADER_FLAG: f'Bearer {auth_key}'
         }
         # Add description in different languages
-        for desc in concept.description:
+        for desc in entity.description:
             if len(desc.content) > 0 and not desc.content == ' ':
                 descriptions.append({
                     DESCRIPTION_TAG: desc.content,
@@ -282,7 +283,7 @@ class WacomKnowledgeService(WacomServiceAPIClient):
                 })
         if len(descriptions) == 0:
             #  Adding an empty description
-            for label in concept.label:
+            for label in entity.label:
                 if len(label.content) > 0 and not label.content == ' ':
                     descriptions.append({
                         DESCRIPTION_TAG: f'Description of {label.content}',
@@ -290,7 +291,7 @@ class WacomKnowledgeService(WacomServiceAPIClient):
                     })
 
         # Labels are tagged as main label
-        for label in concept.label:
+        for label in entity.label:
             if len(label.content) > 0 and label.content != " ":
                 labels.append({
                     'value': label.content,
@@ -298,7 +299,7 @@ class WacomKnowledgeService(WacomServiceAPIClient):
                     'isMain': True
                 })
         # Alias are no main labels
-        for label in concept.alias:
+        for label in entity.alias:
             if len(label.content) > 0 and label.content != " ":
                 labels.append({
                     'value': label.content,
@@ -306,7 +307,7 @@ class WacomKnowledgeService(WacomServiceAPIClient):
                     'isMain': False
                 })
         # Labels are tagged as main label
-        for literal_property, list_literals in concept.data_properties.items():
+        for literal_property, list_literals in entity.data_properties.items():
             for li in list_literals:
                 if li.data_property_type:
                     literals.append({
@@ -315,24 +316,113 @@ class WacomKnowledgeService(WacomServiceAPIClient):
                         DATA_PROPERTY_TAG: li.data_property_type.iri
                     })
         payload: dict = {
-            TYPE_TAG: concept.concept_type.iri,
+            TYPE_TAG: entity.concept_type.iri,
             DESCRIPTIONS_TAG: descriptions,
-            IMAGE_TAG: concept.image,
+            IMAGE_TAG: entity.image,
             LABELS_TAG: labels,
             DATA_PROPERTIES_TAG: literals
         }
-        if concept.tenant_access_right:
-            payload[TENANT_RIGHTS_TAG] = concept.tenant_access_right.to_list()
+        if entity.tenant_access_right:
+            payload[TENANT_RIGHTS_TAG] = entity.tenant_access_right.to_list()
         response: Response = requests.post(url, json=payload, headers=headers, verify=self.verify_calls)
         if response.ok:
             uri: str = response.json()[URI_TAG]
             return uri
 
         import json
-        raise WacomServiceException(f'Pushing concept failed. '
+        raise WacomServiceException(f'Pushing entity failed. '
                                     f'Response code:={response.status_code}, exception:= {response.content}. '
                                     f'Payload: \n{json.dumps(payload, indent=4)}')
 
+    def update_entity(self, auth_key: str, entity: ThingObject):
+        """
+        Updates entity in graph.
+
+        Parameters
+        ----------
+        auth_key: str
+            Auth key from user
+        entity: ThingObject
+            entity object
+
+        Returns
+        -------
+        uri: str
+            URI of entity
+
+        Raises
+        ------
+        WacomServiceException
+            If the graph service returns an error code
+        """
+        uri: str = entity.uri
+        url: str = f'{self.service_base_url}{WacomKnowledgeService.ENTITY_URI_ENDPOINT.format(urllib.parse.quote(uri))}'
+        # Different localized content
+        labels: List[dict] = []
+        descriptions: List[dict] = []
+        literals: List[dict] = []
+        # Header info
+        headers: dict = {
+            'Content-Type': 'application/json',
+            AUTHORIZATION_HEADER_FLAG: f'Bearer {auth_key}'
+        }
+        # Add description in different languages
+        for desc in entity.description:
+            if len(desc.content) > 0 and not desc.content == ' ':
+                descriptions.append({
+                    DESCRIPTION_TAG: desc.content,
+                    LANGUAGE_CODE_TAG: desc.language_code
+                })
+        if len(descriptions) == 0:
+            #  Adding an empty description
+            for label in entity.label:
+                if len(label.content) > 0 and not label.content == ' ':
+                    descriptions.append({
+                        DESCRIPTION_TAG: f'Description of {label.content}',
+                        LANGUAGE_CODE_TAG: label.language_code
+                    })
+
+        # Labels are tagged as main label
+        for label in entity.label:
+            if len(label.content) > 0 and label.content != " ":
+                labels.append({
+                    'value': label.content,
+                    LANGUAGE_CODE_TAG: label.language_code,
+                    'isMain': True
+                })
+        # Alias are no main labels
+        for label in entity.alias:
+            if len(label.content) > 0 and label.content != " ":
+                labels.append({
+                    'value': label.content,
+                    LANGUAGE_CODE_TAG: label.language_code,
+                    'isMain': False
+                })
+        # Labels are tagged as main label
+        for literal_property, list_literals in entity.data_properties.items():
+            for li in list_literals:
+                if li.data_property_type:
+                    literals.append({
+                        'value': li.value,
+                        LANGUAGE_CODE_TAG: li.language_code,
+                        DATA_PROPERTY_TAG: li.data_property_type.iri
+                    })
+        payload: dict = {
+            TYPE_TAG: entity.concept_type.iri,
+            DESCRIPTIONS_TAG: descriptions,
+            IMAGE_TAG: entity.image,
+            LABELS_TAG: labels,
+            DATA_PROPERTIES_TAG: literals
+        }
+        if entity.tenant_access_right:
+            payload[TENANT_RIGHTS_TAG] = entity.tenant_access_right.to_list()
+        response: Response = requests.patch(url, json=payload, headers=headers, verify=self.verify_calls)
+        if not response.ok:
+            import json
+            raise WacomServiceException(f'Pushing entity failed. '
+                                        f'Response code:={response.status_code}, exception:= {response.content}. '
+                                        f'Payload: \n{json.dumps(payload, indent=4)}')
+
     def relations(self, auth_key: str, uri: str) -> Dict[OntologyPropertyReference, ObjectProperty]:
         """
         Retrieve the relations (object properties) of an entity.
@@ -535,7 +625,7 @@ class WacomKnowledgeService(WacomServiceAPIClient):
         }
         params: dict = {
             'uris': uris,
-            'activation': depth
+            ACTIVATION_TAG: depth
         }
 
         response: Response = requests.get(url, headers=headers, params=params, verify=self.verify_calls)
@@ -561,7 +651,7 @@ class WacomKnowledgeService(WacomServiceAPIClient):
         auth_key: str
             Auth key from user
         filter_type: OntologyClassReference
-            Filtering with concept
+            Filtering with entity
         page_id: Optional[str]
             Page id. Start from this page id
         limit: int
@@ -685,7 +775,7 @@ class WacomKnowledgeService(WacomServiceAPIClient):
             LIMIT: limit,
             NEXT_PAGE_ID_TAG: next_page_id
         }
-        url: str = f'{self.service_url}/{self.service_endpoint}{WacomKnowledgeService.SEARCH_TYPES_ENDPOINT}'
+        url: str = f'{self.service_base_url}{WacomKnowledgeService.SEARCH_TYPES_ENDPOINT}'
         response: Response = requests.get(url, headers=headers, params=parameters, verify=self.verify_calls)
         if response.ok:
             return WacomKnowledgeService.__search_results__(response.json())
